/*
      #include "config.h"

      #ifdef EXPLICIT_S
      #include <srfftw.h>
      #else
      #include <rfftw.h>
      #endif //EXPLICIT_S
*/

#include <srfftw.h>

#include "convolve.h"

#include "stk-m1.h"

float unit[] = {1.0f};

rfftw_plan plan_rc[NUM_PLANS],
	   plan_cr[NUM_PLANS];
int plan_length[NUM_PLANS];

fftw_real *impulse_freq;
fftw_real *block_time;
fftw_real *block_freq;
fftw_real *op;
float *opc;
float *overlap;
unsigned int in_ptr;
unsigned int out_ptr;
int fft_length;
int plan;

float peak = 0.0f;

inline void impulse2freq(float *imp, unsigned int length, fftw_real *out)
{
	fftw_real impulse_time[MAX_FFT_LENGTH];
	unsigned int i;
	unsigned int new_fft_length, new_plan;

	printf("++impulse2freq()\n");

	new_fft_length = 0;
	new_plan = 0;
	for (i=0; i<NUM_PLANS; i++) {
		if (plan_length[i] > length + SEG_LENGTH) {
			new_fft_length = plan_length[i];
			new_plan = i;
			printf("picked plan %d, %d bins\n", new_plan, plan_length[i]);
			break;
		}
	}
	if (!new_fft_length) {
		new_fft_length = plan_length[NUM_PLANS-1];
		new_plan = NUM_PLANS-1;
	}

	for (i=0; i<length; i++) {
		impulse_time[i] = imp[i];
	}
	for (; i<fft_length; i++) {
		impulse_time[i] = 0.0f;
	}
	rfftw_one(plan_rc[new_plan], impulse_time, out);

	fft_length = new_fft_length;
	plan = new_plan;
	printf("--impulse2freq()\n");
}

void setup()
{
	int i, len;

	impulse_freq = calloc(MAX_FFT_LENGTH, sizeof(fftw_real));
	block_time = calloc(MAX_FFT_LENGTH, sizeof(fftw_real));
	block_freq = calloc(MAX_FFT_LENGTH, sizeof(fftw_real));
	op = calloc(MAX_FFT_LENGTH, sizeof(fftw_real));
	overlap = calloc(MAX_FFT_LENGTH - SEG_LENGTH, sizeof(float));
	opc = calloc(SEG_LENGTH, sizeof(float));

	for (i=0, len=256; i<NUM_PLANS; i++, len *= 2) {
		plan_rc[i] = rfftw_create_plan(len,
		 FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE);
		plan_cr[i] = rfftw_create_plan(len,
		 FFTW_COMPLEX_TO_REAL, FFTW_ESTIMATE);
		plan_length[i] = len;
	}

	impulse2freq(stk_m1, IMP_LENGTH(stk_m1), impulse_freq);

	in_ptr = 0;
	out_ptr = 0;
}

void run(float *in, float *out, int sample_count)
{
      unsigned long i, pos, ipos, limit;
      fftw_real tmp;
      float coef = 1.0f / (float)fft_length;

      for (pos = 0; pos < sample_count; pos += SEG_LENGTH) {
        limit = pos + SEG_LENGTH;

        for (ipos = pos; ipos < sample_count && ipos<limit; ipos++) {
          block_time[in_ptr++] = in[ipos];

          if (in_ptr == SEG_LENGTH) {
            rfftw_one(plan_rc[plan], block_time, block_freq);
  
            for (i=1; i<(fft_length / 2); i++) {
              tmp = block_freq[i] * impulse_freq[i] -
               block_freq[fft_length-i] * impulse_freq[fft_length-i];
              block_freq[fft_length-i] =
               block_freq[i] * impulse_freq[fft_length-i] +
               block_freq[fft_length-i] * impulse_freq[i];
              block_freq[i] = tmp;
            }
            block_freq[0] = 0.0f;
            block_freq[fft_length / 2] = 0.0f;
      
            rfftw_one(plan_cr[plan], block_freq, op);
      
            for (i=0; i<fft_length-SEG_LENGTH; i++) {
              op[i] += overlap[i];
            }
            for (i=SEG_LENGTH; i<fft_length; i++) {
              overlap[i-SEG_LENGTH] = op[i];
            }

            in_ptr = 0;
          }
        }
  
        for (ipos = pos; ipos < sample_count && ipos<limit; ipos++) {
          out[ipos] = opc[out_ptr++] * coef * 0.5f;
	  if (out[ipos] > peak) {
		peak = out[ipos];
		printf("peak %f\n", peak);
	  }
          if (out_ptr == SEG_LENGTH) {
            for (i=0; i<SEG_LENGTH; i++) {
              opc[i] = op[i];
            }
            out_ptr = 0;
          }
        }
      }
}
