#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include <pthread.h>
#include <gtk/gtk.h>
#include <jack/jack.h>
#include <asm/msr.h>

#include "interface.h"
#include "support.h"
#include "convolve.h"

GtkWidget *window1;
GtkWidget *fileselection1;

jack_client_t *client;

jack_port_t *input_port;
jack_port_t *output_port;

unsigned int impulse_sent = 0;
float *response;
unsigned long response_duration;
unsigned long response_pos;
int grab_finished = 0;

long long last_call = 0;

int
process (nframes_t nframes, void *arg)

{
  sample_t *out = (sample_t *) jack_port_get_buffer (output_port, nframes);
  sample_t *in = (sample_t *) jack_port_get_buffer (input_port, nframes);
  long long then, now;

  rdtscll(then);
  run(in, out, nframes);
  rdtscll(now);
  //printf("%16lld%16lld\n", now-then, then-last_call);
  last_call = then;

  return 0;      
}

void
jack_shutdown (void *arg)
{
  exit (1);
}

int
main (int argc, char *argv[])
{
  gtk_set_locale ();
  gtk_init (&argc, &argv);

  add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
  add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps");

  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  window1 = create_window1 ();
  gtk_widget_show(window1);
  fileselection1 = create_fileselection1 ();

  /* Init data areas for convolver */
  setup();

  /* try to become a client of the JACK server */

  if ((client = jack_client_new("pulsor")) == 0) {
    fprintf (stderr, "jack server not running?\n");
    return 1;
  }

  /* tell the JACK server to call `process()' whenever
     there is work to be done.
  */

  jack_set_process_callback(client, process, 0);

  /* tell the JACK server to call `jack_shutdown()' if
     it ever shuts down, either entirely, or if it
     just decides to stop calling us.
  */

  jack_on_shutdown(client, jack_shutdown, 0);

  input_port = jack_port_register(client, "input", JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
  output_port = jack_port_register(client, "output", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);

  /* tell the JACK server that we are ready to roll */

  if (jack_activate(client)) {
    fprintf(stderr, "cannot activate client");
    return 1;
  }

  /* connect the ports. Note: you can't do this before
     the client is activated (this may change in the future).
  */

  if (jack_connect(client, "alsa_pcm:in_1", jack_port_name (input_port))) {
    fprintf(stderr, "cannot connect input ports\n");
  }
  
  if (jack_connect(client, jack_port_name (output_port), "alsa_pcm:out_1")) {
    fprintf(stderr, "cannot connect output ports\n");
  }

  gtk_main();

  jack_client_close (client);

  return 0;
}

